/* eslint-env mocha */

'use strict'

let babel = require('..')
let chai = require('chai')
let convert = require('convert-source-map')
let js = require('mako-js')
let mako = require('mako')
let path = require('path')
let buffer = require('mako-buffer')
let vm = require('vm')

chai.use(require('chai-as-promised'))
let assert = chai.assert
let fixture = path.resolve.bind(path, __dirname, 'fixtures')

describe('babel plugin', function () {
  it('should transpile the file', function () {
    let entry = fixture('simple/index.js')
    let runner = mako().use(buffer('js'), babel(), js())

    return runner.build(entry).then(function (build) {
      let file = build.tree.findFile(entry)
      assert.deepEqual(exec(file), { a: 1 })
    })
  })

  it('should ignore node_modules by default', function () {
    let entry = fixture('ignore/index.js')
    let runner = mako().use(buffer('js'), babel(), js())

    // since the node_module was not complied, it should have a syntax error
    return assert.isRejected(runner.parse(entry))
  })

  context('with options', function () {
    context('.extensions', function () {
      it('should convert other file types into js', function () {
        let entry = fixture('extensions/index.es')
        let runner = mako().use(buffer('es'), babel({ extensions: 'es' }), js())

        return runner.build(entry).then(function (build) {
          let file = build.tree.findFile(entry)
          assert.equal(file.type, 'js')
        })
      })
    })

    context('.sourceMaps', function () {
      it('should include an inline source-map', function () {
        let entry = fixture('simple/index.js')
        let runner = mako().use(buffer('js'), babel({ sourceMaps: true }), js())

        return runner.parse(entry).then(function (build) {
          let file = build.tree.findFile(entry)
          assert.isTrue(convert.commentRegex.test(file.contents))
        })
      })
    })
  })
})

/**
 * Execute the JS string in a new context
 *
 * @param  {String} file  The build file to execute.
 * @return {Mixed} value
 */
function exec (file) {
  const id = path.relative(file.base, file.initialPath)
  return vm.runInNewContext(file.contents)(id)
}
